﻿--[[
Baggins_ClosetGnome

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Nargiddley for writing Baggins and Rabbit for writing ClosetGnome

License

Copyright (c) 2007 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

Baggins_ClosetGnome = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceEvent-2.0")

local Baggins_ClosetGnome = Baggins_ClosetGnome

local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Baggins_ClosetGnome")
local items_table = {}
local sets_table = {}

L:RegisterTranslations("enUs", function()
	return {
		["ClosetGnome Set:%s"] = true,
		["Unknown"] = true,
		["ClosetGnome Sets"] = true,
		["Any"] = true,
		["None"] = true,
		["ClosetGnome"] = true,
		["Filter by ClosetGnome set"] = true,
		["Fuzzy match"] = true,
		["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

-- Feel free to fix any of the localizations without asking first and check them into the SVN.
L:RegisterTranslations("esES", function()
	return {
		--["ClosetGnome Set:%s"] = true,
		--["Unknown"] = true,
		--["ClosetGnome Sets"] = true,
		--["Any"] = true,
		["None"] = "Ninguno", -- check
		--["ClosetGnome"] = true,
		--["Filter by ClosetGnome set"] = true,
		--["Fuzzy match"] = true,
		--["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

L:RegisterTranslations("deDE", function()
	return {
		--["ClosetGnome Set:%s"] = true,
		["Unknown"] = "Unbekannt", -- check
		--["ClosetGnome Sets"] = true,
		--["Any"] = true,
		["None"] = "Keine",
		--["ClosetGnome"] = true,
		--["Filter by ClosetGnome set"] = true,
		--["Fuzzy match"] = true,
		--["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

L:RegisterTranslations("frFR", function()
	return {
		--["ClosetGnome Set:%s"] = true,
		["Unknown"] = "Inconnu", -- check
		--["ClosetGnome Sets"] = true,
		--["Any"] = true,
		["None"] = "Aucun", -- check
		--["ClosetGnome"] = true,
		--["Filter by ClosetGnome set"] = true,
		--["Fuzzy match"] = true,
		--["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

L:RegisterTranslations("koKR", function()
	return {
		["ClosetGnome Set:%s"] = "ClosetGnome ~~~:%s",
		["Unknown"] = "~~~~~", -- check
		["ClosetGnome Sets"] = "ClosetGnome ~~",
		["Any"] = "~~",
		["None"] = "~~", -- check
		--["ClosetGnome"] = true,
		["Filter by ClosetGnome set"] = "ClosetGnome ~~~ ~~ ~~~",
		["Fuzzy match"] = "~~~~ ~~",
		["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "~~~~~ ~~ ~~~~ ~~~~ ~~~ ID~ ~~ ~~~~~",
	};
end)

L:RegisterTranslations("znCN", function()
	return {
		--["ClosetGnome Set:%s"] = true,
		["Unknown"] = "~~", -- check
		--["ClosetGnome Sets"] = true,
		--["Any"] = true,
		["None"] = "~", -- check
		--["ClosetGnome"] = true,
		--["Filter by ClosetGnome set"] = true,
		--["Fuzzy match"] = true,
		--["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

L:RegisterTranslations("znTW", function()
	return {
		--["ClosetGnome Set:%s"] = true,
		["Unknown"] = "~~", -- check
		--["ClosetGnome Sets"] = true,
		--["Any"] = true,
		["None"] = "~", -- check
		--["ClosetGnome"] = true,
		--["Filter by ClosetGnome set"] = true,
		--["Fuzzy match"] = true,
		--["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
	};
end)

L:RegisterTranslations("ruRU", function()
	return {
		["ClosetGnome Set:%s"] = "Набор ClosetGnome:%s",
		["Unknown"] = "Неизвестно",
		["ClosetGnome Sets"] = "Наборы ClosetGnome",
		["Any"] = "Любой",
		["None"] = "Ни один",
		["ClosetGnome"] = "ClosetGnome",
		["Filter by ClosetGnome set"] = "Фильтр по наборам ClosetGnome",
		["Fuzzy match"] = "Нечёткое соответствие",
		["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "Соответствие по номеру вещи, игнорирование зачарований, вариаций, коплектов брони и т.п.",
	};
end)

-- Prototypes
local linkToId -- (link)
local idToFuzzy -- (id)
local Matches -- (bag,slot,rule)
local GetName -- (rule)
local CleanRule -- (rule)
local setRuleSet -- (rule, set)
local toggleRuleFuzzy -- (rule)
local DewDropOptions -- (rule, level, value)
local CacheSets -- ()

local function linkToId(link)
	if type(link) == "string" then
		local first,last = link:match("item:(%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:)(%-?%d+)")
		if first and last then
			last = last % 65536
			return first..last
		end
	end
	return nil
end

local function idToFuzzy(id)
	if type (id) == "string" then
		return id:match("(%-?%d+)")
	else
		return id
	end
end

local function Matches(bag,slot,rule)
	local id = linkToId(GetContainerItemLink(bag, slot))
	if not id then return end

	local set = rule.set
	if not set then return end

	if rule.fuzzy then
		id = idToFuzzy(id)
		if not id then return end
	end

	local res = items_table[id]
	if set == "None" then
		res = not res
	elseif set ~= "Any" then
		res = res and res[set]
	end
	return res
end

local function GetName(rule)
	local msg
	msg = L["ClosetGnome Set:%s"]:format(rule.set or L["Unknown"])
	if rule.fuzzy then
		msg = msg.." "..L["Fuzzy match"]
	end
	return msg
end

local function CleanRule(rule)
	rule.set = "Any"
	rule.fuzzy = nil
end

local function setRuleSet(rule, set)
	rule.set = set
	Baggins:OnRuleChanged()
end

local function toggleRuleFuzzy(rule)
	rule.fuzzy = not rule.fuzzy
	Baggins:OnRuleChanged()
end

local function DewDropOptions(rule, level, value)
	if level == 1 then
		dewdrop:AddLine("text", L["ClosetGnome Sets"], "isTitle", true)
		dewdrop:AddLine('text', L["Fuzzy match"], 'checked', rule.fuzzy, 'func',toggleRuleFuzzy,'arg1',rule,
				'tooltipTitle', L["Fuzzy match"],
				'tooltipText',L["Match by the item number, ignoring enchantments, variations, armor kits, etc."])
		dewdrop:AddLine("text", L["Any"], "checked", rule.set == "Any",'func',setRuleSet,'arg1',rule,'arg2',"Any")
		dewdrop:AddLine("text", L["None"], "checked", rule.set == "None",'func',setRuleSet,'arg1',rule,'arg2',"None")
		local setnames = {}
		for set in pairs(sets_table) do
			table.insert(setnames,set)
		end
		table.sort(setnames)
		for _,set in ipairs(setnames) do
			dewdrop:AddLine('text', set, "checked", rule.set == set, 'func',setRuleSet,'arg1',rule,'arg2',set)
		end
	end
end

local function CacheSets()
	local i,set

	local sets = ClosetGnome.db.char.set

	items_table = {}
	sets_table = {}
	for setname, set in pairs(sets) do
		sets_table[setname] = true
		for i=0,19 do
			local setitem = linkToId(set[i])
			if setitem then
				if not items_table[setitem] then items_table[setitem] = {} end
				items_table[setitem][setname] = true
				local fuzzy = idToFuzzy(setitem)
				if fuzzy then
					if not items_table[fuzzy] then items_table[fuzzy] = {} end
					items_table[fuzzy][setname] = true
				end
			end
		end
	end
end

function Baggins_ClosetGnome:OnInitialize()
	CacheSets()
	Baggins:AddCustomRule("ClosetGnome", {
		DisplayName = L["ClosetGnome"],
		Description = L["Filter by ClosetGnome set"],
		Matches = Matches,
		GetName = GetName,
		CleanRule = CleanRule,
		DewDropOptions = DewDropOptions
	})
end

function Baggins_ClosetGnome:OnEnable()
	--CacheSets()
	self:RegisterEvent("ClosetGnome_AddSet", "CacheSetsAndUpdate")
	self:RegisterEvent("ClosetGnome_UpdateSet", "CacheSetsAndUpdate")
	self:RegisterEvent("ClosetGnome_DeleteSet", "CacheSetsAndUpdate")
	--Baggins:ForceFullUpdate()
end

function Baggins_ClosetGnome:CacheSetsAndUpdate()
	CacheSets()
	Baggins:ForceFullUpdate()
end
